﻿local L = AceLibrary("AceLocale-2.2"):new("GridAutoFrameSize")
local GridL = AceLibrary("AceLocale-2.2"):new("Grid")
local BZ = LibStub("LibBabble-Zone-3.0"):GetReverseLookupTable()

local math_ceil = math.ceil
local math_max = math.max
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local GetRaidRosterInfo = GetRaidRosterInfo
local GetInstanceDifficulty = GetInstanceDifficulty

GridAutoFrameSize = Grid:NewModule("GridAutoFrameSize")
local GridAutoFrameSize = GridAutoFrameSize

GridAutoFrameSize.defaultDB = {
	autoSizeFrames = 1,
	autoZoneSize = 0,
	zoneLock = 0, -- set to 1 when the raid frame is set by zone and not to be changed by raid size
	withPets = false,
}

local battlegrounds = {
  ["Alterac Valley"] = 40,
  ["Arathi Basin"] = 15,
  ["Eye of the Storm"] = 15,
  ["Strand of the Ancients"] = 15,
  ["Warsong Gulch"] = 10,
}

local instances = {
  ["Naxxramas"] = true,
  ["The Eye of Eternity"] = true,
  ["The Obsidian Sanctum"] = true,
  ["Vault of Archavon"] = true,
}

local function getMode()
  local mode = GetInstanceDifficulty()
  if mode == 1 then
    return 10
  elseif mode == 2 then
    return 25
  else
    return 0
  end
end

local function getPredefinedZoneSize(zone)
  local englishZone = BZ[zone]

  if battlegrounds[englishZone] then
    return battlegrounds[englishZone]
  end
  
  if instances[englishZone] then
    return getMode()
  end
  
  return 0
end

function GridAutoFrameSize:isWithPets()
	return self.db.profile.withPets
end

function GridAutoFrameSize:OnEnable()
	local opt = GridLayout.options.args
	opt["autoSizeFrames"] = {
		type = "toggle",
		name = L["AUTO_SIZE"],
		desc = L["AUTO_SIZE_DESC"],
		order = 28,
		get = function ()	return self.db.profile.autoSizeFrames	end,
		set = function (v)
			self.db.profile.autoSizeFrames = v
			self:CheckRoster()
		end,	
	}
	opt["autoZoneSize"] = {
		type = "toggle",
		name = L["ZONE_SIZE"],
		desc = L["ZONE_SIZE_DESC"],
		order = 29,
		get = function ()	return self.db.profile.autoZoneSize end,
		set = function (v)
			self.db.profile.autoZoneSize = v
			self.db.profile.zoneLock = 0
			self:CheckRoster()
		end,	
	}
	opt["withPets"] = {
		type = "toggle",
		name = L["WITH_PETS"],
		desc = L["WITH_PETS_DESC"],
		order = 30,
		get = function() return self.db.profile.withPets end,
		set = function() 
			self.db.profile.withPets = not self.db.profile.withPets
			self:CheckRoster()
		end,
	}
	
	self.db.profile.zoneLock = 0
	self:RegisterBucketEvent({"RAID_ROSTER_UPDATE", "PARTY_MEMBERS_CHANGED"}, 1, "CheckRoster")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckZone")		
end

function GridAutoFrameSize:CheckZone()
	local zone = GetRealZoneText()
  local size = getPredefinedZoneSize(zone)
	if size ~= 0 then
		self.db.profile.zoneLock = 1
		self:Debug("frame size zone locked to : "..zone.." - "..size)
		
		local newLayout = "By Group "..size
		if size ~= 40 then
			if self:isWithPets() then
				newLayout = newLayout.." w/Pets"
			end
		end
		
		newLayout = newLayout and GridL[newLayout]
		
		if not newLayout or newLayout == GridLayout.db.profile.layout then
			self:Debug("no layout change")		
			return
		end				
		self:Debug("new layout: "..newLayout)		
		GridLayout.db.profile.layout = newLayout
		GridLayout:LoadLayout(newLayout)		
	else
		self.db.profile.zoneLock = 0
		self:Debug("zone unlocked")
	end
	self:Debug("zoned into : "..zone)
end

function GridAutoFrameSize:CheckRoster()
	if ( (not self.db.profile.autoSizeFrames) or (self.db.profile.zoneLock == 1) ) then
		if (self.db.profile.zoneLock == 1) then
			self:Debug("zone lock, cant change !")
		end
		return
	end
	
	local newLayout
	local raidSize = GetNumRaidMembers()
	if raidSize > 0 then
		
		local maxGroup = math_ceil(raidSize / 5)
		for i = 1, raidSize do
			local group = select(3, GetRaidRosterInfo(i))
			maxGroup = math_max(maxGroup, group)
		end
		if (maxGroup > 5) or (raidSize > 25) then
			newLayout = "By Group 40"
		else
			if 5 * maxGroup == 20 then
				newLayout = "By Group 25"
			else
				newLayout = "By Group " .. 5 * maxGroup
			end

			if self:isWithPets() then
				newLayout = newLayout.." w/Pets"
			end
		end
		self:Debug(("raid settings: size=%d, maxgroup=%d => layout=%s"):format(raidSize, maxGroup, newLayout))

	elseif GetNumPartyMembers() > 0 then
		newLayout = "By Group 5"
		if self:isWithPets() then
			newLayout = newLayout.." w/Pets"
		end
		self:Debug(("in party or not grouped => layout=%s"):format(newLayout))
	end
	
	newLayout = newLayout and GridL[newLayout]
	
	if not newLayout or newLayout == GridLayout.db.profile.layout then
		self:Debug("no layout change")		
		return
	end
	
	self:Debug("new layout: "..newLayout)		
	GridLayout.db.profile.layout = newLayout
	GridLayout:LoadLayout(newLayout)	
end
